@@ -16,6 +16,8 @@ __author__ = "Andrea Tramacere"
 # Project
 # relative import eg: from .mod import f
 
+import typing
+
 from json_tricks import numpy_encode,dumps,loads,numeric_types_hook,hashodict,json_numpy_obj_hook
 from astropy.io import fits as pf
 from astropy.io import ascii as astropy_io_ascii
@@ -31,16 +33,21 @@ import  pickle
 import gzip
 import  hashlib
 from numpy import nan,inf
-from sys import version_info
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
+from sys import path_importer_cache, version_info
 
+from io import StringIO
+
+import logging
+
+logger = logging.getLogger('oda_api.data_products')
 
 __all__=['sanitize_encoded','_chekc_enc_data','BinaryData','NumpyDataUnit','NumpyDataProduct','ApiCatalog','ODAAstropyTable']
 
+import astropy.io.fits.fitsrec
+import numpy as np
 
+
+# these 3 functions are remnants of misusing repr() to serialize data instead of json
 def sanitize_encoded(d):
     d = d.replace('null', 'None')
     d = d.replace('true', 'True')
@@ -49,6 +56,21 @@ def sanitize_encoded(d):
     d = d.replace('Infinity', 'inf')
     return d
 
+def json_to_literal(d):
+    d = d.replace('null', 'None')
+    d = d.replace('true', 'True')
+    d = d.replace('false', 'False')
+    d = d.replace('NaN', 'nan')
+    d = d.replace('Infinity', 'inf')
+    return d
+
+def literal_to_json(d):
+    d = d.replace('None', 'null')
+    d = d.replace('True', 'true')
+    d = d.replace('False', 'false')
+    d = d.replace('nan', 'NaN')
+    d = d.replace('inf', 'Infinity')
+    return d
 
 
 def _chekc_enc_data(data):
@@ -59,10 +81,7 @@ def _chekc_enc_data(data):
 
     return _l
 
-
-
-
-
+# not used?
 class ODAAstropyTable(object):
 
     def __init__(self,table_object,name='astropy table', meta_data={}):
@@ -160,14 +179,14 @@ class BinaryData(object):
 
 class NumpyDataUnit(object):
 
-    def __init__(self,data,data_header={},meta_data={},hdu_type=None,name='table',units_dict=None):
+    def __init__(self, data, data_header={}, meta_data={}, hdu_type=None, name='table', units_dict=None):
         self._hdu_type_list_ = ['primary', 'image', 'table', 'bintable']
 
         self.name=name
-        self._chekc_data(data)
-        self._chekc_hdu_type(hdu_type)
-        self._chekc_dict(data_header)
-        self._chekc_dict(meta_data)
+        self._check_data(data)
+        self._check_hdu_type(hdu_type)
+        self._check_dict(data_header)
+        self._check_dict(meta_data)
 
         self.data=data
         self.header=data_header
@@ -175,25 +194,37 @@ class NumpyDataUnit(object):
         self.hdu_type=hdu_type
         self.units_dict=units_dict
 
-    def _chekc_data(self,data):
+    # interface with a typo, preserving with a warning
+    def _warn_chekc_typo(self):
+        logger.debug('please _check_* instead of _chekc_* functions, they will be removed')
+    
+    def _chekc_data(self, data):
+        self._warn_chekc_typo()
+        return self._check_data(data)        
+
+    def _chekc_hdu_type(self,hdu_type):
+        return self._check_hdu_type(hdu_type)
+
+    def _chekc_dict(self, _kw):
+        self._warn_chekc_typo()
+        return self._check_dict(_kw)
 
 
+    def _check_data(self,data):
         if isinstance(data, numpy.ndarray) or data is None:
             pass
         else:
             raise RuntimeError('data is not numpy ndarray object')
 
-    def _chekc_hdu_type(self,hdu_type):
+    def _check_hdu_type(self,hdu_type):
         if hdu_type is None:
             pass
         elif hdu_type in self._hdu_type_list_:
             pass
         else:
             raise RuntimeError('hdu type ', hdu_type, 'not in allowed', self._hdu_type_list_)
-
-
-
-    def _chekc_dict(self,_kw):
+    
+    def _check_dict(self,_kw):
 
         if isinstance(_kw, dict):
             pass
@@ -298,18 +329,22 @@ class NumpyDataUnit(object):
 
             if use_pickle is True:
 
+                if isinstance(self.data, astropy.io.fits.fitsrec.FITS_rec):
+                    pickled_data = pickle.dumps(self.data)
+                else:
+                    pickled_data = pickle.dumps(numpy.array(self.data))
+
                 if use_gzip==True:
-                    print ('gizziping')
-                    out_file = StringIO()
+                    out_file = StringIO(pickled_data)
                     gzip_file = gzip.GzipFile(fileobj=out_file, mode='wb')
 
-
-                    gzip_file.write(pickle.dumps(numpy.array(self.data)))
+                    gzip_file.write(pickled_data)
                     _binarys = base64.b64encode(out_file.getvalue())
                     gzip_file.close()
                 else:
-                    _binarys= base64.b64encode(pickle.dumps(numpy.array(self.data),2))
-
+                    _binarys= base64.b64encode(
+                        pickled_data
+                    )
 
             else:
                 _d= json.dumps(self.data, cls=JsonCustomEncoder)
@@ -371,8 +406,7 @@ class NumpyDataUnit(object):
                     _data = pickle.loads(_binarys)
 
         elif encoded_data is not None:
-            #print('using JsonCustomEncoder')
-            encoded_data=eval(encoded_data)
+            encoded_data=eval(encoded_data) # !!
 
             for ID,c in enumerate(encoded_data):
                 encoded_data[ID]=tuple(c)
@@ -415,14 +449,14 @@ class NumpyDataProduct(object):
         print ('------------------------------')
 
 
-    def get_data_unit(self, ID):
+    def get_data_unit(self, ID: int) -> NumpyDataUnit:
         try:
             return self.data_unit[ID]
         except IndexError as e:
             raise RuntimeError(f"problem get_data_unit ID:{ID} in self.data_unit:{self.data_unit}")
 
-    def get_data_unit_by_name(self,name):
-        _du=None
+    def get_data_unit_by_name(self, name: str) -> typing.Union[NumpyDataUnit, None]:
+        _du = None
 
         for du in self.data_unit:
             if du.name == name:
@@ -443,8 +477,8 @@ class NumpyDataProduct(object):
         else:
             _dl = [data]
 
-        for ID,_d  in enumerate(_dl):
-            if isinstance(_d,NumpyDataUnit):
+        for ID, _d  in enumerate(_dl):
+            if isinstance(_d, NumpyDataUnit):
                 pass
             else:
                 raise RuntimeError ('DataUnit not valid')
@@ -465,13 +499,19 @@ class NumpyDataProduct(object):
 
 
 
-    def encode(self,use_pickle=True,use_gzip=False,to_json=False):
-        _enc=[]
-        #print('use_gzip',use_gzip)
-        for ID, ed in enumerate(self.data_unit):
-            #print('data_unit',ID)
-            _enc.append(self.data_unit[ID].encode(use_pickle=use_pickle,use_gzip=use_gzip))
+    def encode(self, use_pickle=True, use_gzip=False, to_json=False):
+        _enc = []
+
+        for ID, data_unit in enumerate(self.data_unit):
+            _enc.append(
+                data_unit.encode(
+                        use_pickle=use_pickle,
+                        use_gzip=use_gzip
+                    )
+                )
+
         _o_dict={'data_unit_list':_enc,'name':self.name,'meta_data':dumps(self.meta_data)}
+
         if to_json==True:
             return json.dumps(_o_dict)
 
@@ -509,17 +549,30 @@ class NumpyDataProduct(object):
         return cls(data_unit=[NumpyDataUnit.from_fits_hdu(h) for h in  hdul],meta_data=meta_data,name=name)
 
     @classmethod
-    def decode(cls,encoded_obj,from_json=False):
+    def decode(cls, encoded_obj: typing.Union[str, dict], from_json=False):
         if encoded_obj is not None:
-            if from_json==False:
-                try:
-                    encoded_obj = json.loads(sanitize_encoded(encoded_obj))
-                except:
-                    pass
+            # from_json has the opposite meaning of what the name implies
+            obj_dict: dict 
+            if from_json:
+                if isinstance(encoded_obj, dict):
+                    obj_dict = encoded_obj
+                else:
+                    logger.warning('decoding from unexpected object')
+                    obj_dict = encoded_obj # type: ignore
+            else:
+                if isinstance(encoded_obj, dict):
+                    obj_dict = encoded_obj
+                else:
+                    logger.warning('decoding from unexpected object')
+                    try:
+                        obj_dict = json.loads(literal_to_json(encoded_obj))
+                    except Exception as e:
+                        logger.debug('unable to decode json object: %s', e)                    
+                        # why not raise here?                
 
-            encoded_data_unit_list = encoded_obj['data_unit_list']
-            encoded_name = encoded_obj['name']
-            encoded_meta_data = encoded_obj['meta_data']
+            encoded_data_unit_list = obj_dict['data_unit_list']
+            encoded_name = obj_dict['name']
+            encoded_meta_data = obj_dict['meta_data']
 
             _data_unit_list=[]
             #print('encoded_data_unit_list',encoded_data_unit_list)
@@ -530,8 +583,6 @@ class NumpyDataProduct(object):
             encoded_name=None
             encoded_meta_data={}
 
-
-
         return cls(data_unit=_data_unit_list,name=encoded_name,meta_data=eval(encoded_meta_data))
 
 
