@@ -51,7 +51,7 @@ def pick_scw(kind="any"):
     if kind == "crab":
         return "066500220010.001"
     elif kind == "any":
-        scwlist = requests.get("https://www.astro.unige.ch/cdci/astrooda/dispatch-data/"
+        scwlist = requests.get("https://www.astro.unige.ch/mmoda/dispatch-data/"
                                "gw/timesystem/api/v1.0/scwlist/cons/"
                                "2002-12-17T08:00:00/2020-12-21T08:00:00"
                                "?&ra=83&dec=22&radius=200.0&min_good_isgri=1000").json()
@@ -153,10 +153,11 @@ def test_waiting(scw_kind, platform):
 
 
 
-def test_unauthorized(dispatcher_live_fixture):
+def test_unauthorized(dispatcher_api):
     from oda_api.api import UserError
 
-    disp = get_disp(wait=True, platform=dispatcher_live_fixture)
+    disp = dispatcher_api
+    disp.wait=True
 
     with pytest.raises(UserError):
         disp.poll()
@@ -199,6 +200,8 @@ def test_token_expired(dispatcher_live_fixture):
 
     encoded_token = jwt.encode(token_payload, secret_key, algorithm='HS256')
 
+    if isinstance(encoded_token, bytes):
+        encoded_token = encoded_token.decode()
 
     try:        
         data = disp.get_product(
@@ -212,8 +215,7 @@ def test_token_expired(dispatcher_live_fixture):
 
         raise RuntimeError('did not raise Unauthorized for expired token')
     except Unauthorized as e:
-        assert e.message == "RequestNotAuthorized():token expired"
-    
+        assert e.message == "RequestNotAuthorized():the token provided is expired, please try to logout and login again"
 
 
 @pytest.mark.slow
@@ -380,10 +382,10 @@ def test_bad_request(platform):
             )
 
 
-def test_reusing_disp_instance(dispatcher_live_fixture):
+def test_reusing_disp_instance(dispatcher_api):
 
-    disp = get_disp(wait=True, platform=dispatcher_live_fixture)
-    assert disp.wait
+    disp = dispatcher_api
+    disp.wait = True
 
     assert disp.job_id is None
     assert disp.query_status == 'not-prepared'
